﻿using UnityEngine;
using System.Collections;

public class Player : MonoBehaviour {

	bool grounded = false;

	Vector2 velocity = new Vector2(0, 0);
	float acceleration = 1.25f;
	float friction = 3.1f;
	Vector2 velocityMax = new Vector2(0.105f, 0.35f);

	float fallingTime = 1;

	BoxCollider2D myColliderFoot;
	BoxCollider2D myColliderHead;
	BoxCollider2D myColliderBody;

	SpriteRenderer mySpriteRenderer;

	ScoreKeeper scoreKeeper;

	public AudioClip audioDied;
	public AudioClip[] audioGrunts;

	public int platformsCollidingWith = 0;

	public Sprite spritePlayerIdle;
	public Sprite spritePlayerJump;
	public Sprite[] spritePlayerWalk;
	public Sprite spritePlayerDead;

	public GameOver gameOver;

	public bool isPlayerDead = false;

	float deadTime = 0;

	const float walkTimeReset = .075f;
	int walkIndex = 0;
	float walkTime = walkTimeReset;

	// Use this for initialization
	void Start ()
	{
		Transform fcTransform = transform.FindChild("Foot Collider");
		myColliderFoot = fcTransform.gameObject.GetComponent<BoxCollider2D>();

		Transform hcTransform = transform.FindChild ("Head Collider");
		myColliderHead = hcTransform.gameObject.GetComponent<BoxCollider2D>();
		
		Transform bcTransform = transform.FindChild ("Body Collider");
		myColliderBody = bcTransform.gameObject.GetComponent<BoxCollider2D>();
		
		mySpriteRenderer = gameObject.GetComponentInChildren<SpriteRenderer>();

		scoreKeeper = GameObject.Find (".ScoreKeeper").GetComponent<ScoreKeeper>();
	}


	// Update is called once per frame
	void Update ()
	{
		if( !isPlayerDead ) {
			if( Input.GetKey(KeyCode.A) || Input.GetKey (KeyCode.LeftArrow) ) {
				velocity.x -= 3.25f * Time.deltaTime;
				if( transform.localScale.x > 0 ) {
					transform.localScale = new Vector3(-1, 1, 1);
				}
				if( velocity.x < -velocityMax.x ) {
					velocity.x = -velocityMax.x;
				}
			}
			if( Input.GetKey(KeyCode.D) || Input.GetKey (KeyCode.RightArrow) ) {
				velocity.x += 3.25f * Time.deltaTime;
				if( transform.localScale.x < 0 ) {
					transform.localScale = new Vector3(1, 1, 1);
				}
				if( velocity.x > velocityMax.x ) {
					velocity.x = velocityMax.x;
				}
			}

			if( (grounded || fallingTime < 0.0375f) && Input.GetKey (KeyCode.Space) ) {
				int which = Random.Range (0, audioGrunts.Length);
				SoundManager.instance.Play (audioGrunts[which]);

				velocity.y += .30f;
				grounded = false;
				fallingTime = 1;

				mySpriteRenderer.sprite = spritePlayerJump;
			}
			if( !grounded ) {
				fallingTime += Time.deltaTime;
			}
			else {
				if( velocity.x != 0 ) {
					walkTime -= Time.deltaTime;
					if( walkTime <= 0 ) {
						walkTime = walkTimeReset;
						walkIndex = (walkIndex + 1) % spritePlayerWalk.Length;
						mySpriteRenderer.sprite = spritePlayerWalk[walkIndex];
					}
				}
				else {
					mySpriteRenderer.sprite = spritePlayerIdle;
				}
			}
		}
		else {
			deadTime += Time.deltaTime;

			if( deadTime > 1 ) {
				if( Input.anyKey ) {
					Application.LoadLevel ("title");
				}
			}
		}
	}

	void FixedUpdate()
	{
		if( !grounded ) {
			if( velocity.y > -velocityMax.y) {
				velocity.y -= acceleration * Time.deltaTime;
			}
		}
		
		transform.position = new Vector3(transform.position.x + velocity.x, transform.position.y + velocity.y, 0);
		
		if( velocity.x < 0 ) {
			velocity.x += friction * Time.deltaTime;
			if( velocity.x > -0.001f ) {
				velocity.x = 0;
			}
		}
		else {
			velocity.x -= friction * Time.deltaTime;
			if( velocity.x < 0.001f ) {
				velocity.x = 0;
			}
		}
	}

	void OnCollisionEnter2D( Collision2D collision )
	{
		if( isPlayerDead ) {
			return;
		}

		float collisionDepth = 0;

		BoxCollider2D collider;
		if( myColliderFoot.IsTouching(collision.collider) ) {
			collider = myColliderFoot;

			float itTop = collision.collider.bounds.center.y + collision.collider.bounds.extents.y / 2;
			float myBtm = collider.bounds.center.y - collision.collider.bounds.extents.y / 2;
			collisionDepth = myBtm - itTop;
		}
		else if( myColliderHead.IsTouching (collision.collider) ){
			collider = myColliderHead;

			float itBtm = collision.collider.bounds.center.y - collision.collider.bounds.extents.y / 2;
			float myTop = collider.bounds.center.y + collision.collider.bounds.extents.y / 2;
			collisionDepth = itBtm - myTop;
		}
		else {
			collider = myColliderBody;
		}

		if( collision.gameObject.name == "Platform Collider" ) {
			if( collider == myColliderFoot ) {
				if( velocity.y < 0 && !isPlayerDead) {
					// we're falling down
					grounded = true;
					mySpriteRenderer.sprite = spritePlayerIdle;
					velocity.y = 0;
					transform.position += new Vector3(0, (collisionDepth * -1) + collider.bounds.extents.y, 0);
				}
			}
			else if( collider == myColliderHead ) {
				if( velocity.y > 0 ) {
					Platform platform = collision.gameObject.GetComponentInParent<Platform>();
					if( platform && platform.solidBottom ) {
						velocity.y = 0;
						transform.position -= new Vector3(0, (collisionDepth * -1) + collider.bounds.extents.y, 0);
					}
				}
			}

			platformsCollidingWith += 1;
		}
		else if( collision.gameObject.name == "Hazard Collider" && !isPlayerDead ) {
			//mySpriteRenderer.material.color = new Color(1,.25f,.25f,1);
			mySpriteRenderer.sprite = spritePlayerDead;
			isPlayerDead = true;
			SoundManager.instance.Play (audioDied, true);
			gameOver.Trigger();
			velocity.x = 0;
			velocity.y = .315f;
		}
		else if( collision.gameObject.name == "Bonus Collider" && !isPlayerDead && collider != myColliderHead ) {
			Bonus bonus = collision.gameObject.GetComponentInParent<Bonus>();
			if( bonus ) {
				SoundManager.instance.Play (bonus.audio, true);
				Destroy (bonus.gameObject);
				scoreKeeper.AddScore(300);
			}
		}

		if( collision.gameObject.name == "Warp.Left" && myColliderBody.IsTouching (collision.collider) ) {
			gameObject.transform.position = new Vector3(-gameObject.transform.position.x, gameObject.transform.position.y, gameObject.transform.position.z);
		}
		else if( collision.gameObject.name == "Warp.Right" && myColliderBody.IsTouching (collision.collider) ) {
			gameObject.transform.position = new Vector3(-gameObject.transform.position.x, gameObject.transform.position.y, gameObject.transform.position.z);
		}
	}

	void OnCollisionExit2D( Collision2D collision )
	{
		if( collision.gameObject.name == "Platform Collider" ) {
			platformsCollidingWith -= 1;
			if( platformsCollidingWith == 0 ) {
				grounded = false;
				if( velocity.y == 0 ) {
					fallingTime = 0;
				}
			}
		}
	}
}
