﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class WorldGen : MonoBehaviour {


	[System.Serializable]
	public struct ArrangementInfo
	{
		public enum Type
		{
			Easy,
			Medium,
			Hard,
			Brutal,
			Impossible
		}

		public Type type;
		public int levels;
		public GameObject prefabArrangement;
	}
	

	public ArrangementInfo[] arrangements;
	public GameObject cameraObj;
	public GameObject prefabPlatformStandard;
	public GameObject prefabBackground;
	public AudioClip audioSoundtrack;

	int screenUnits = 16;
	float platformDistance = 1.5f;

	public float currentLevel = 0;
	float startHeight = -2.5f;
	float backgroundUnits = 0;

	Queue<GameObject> usedArrangements = new Queue<GameObject>();

	public GameObject[] backgrounds = new GameObject[2];

	// Use this for initialization
	void Start () {
		// create backgrounds
		backgrounds[0] = (GameObject)Instantiate (prefabBackground, new Vector3(0,0,10), Quaternion.identity);
		backgrounds[1] = (GameObject)Instantiate (prefabBackground, new Vector3(0,0,10), Quaternion.identity);

		backgroundUnits = backgrounds[0].GetComponent<SpriteRenderer>().bounds.extents.y * 2;
		backgrounds[1].transform.position += new Vector3(0, backgroundUnits, 0);

		Debug.Log ("backgroundUnits = " + backgroundUnits);

		// always start with a solid base
		if( prefabPlatformStandard != null ) {
			float screenLeft = -(screenUnits / 2.0f);

			float x = screenLeft + .5f;
			for( int i = 0; i < screenUnits; ++i ) {
				Instantiate(prefabPlatformStandard, new Vector3(x, startHeight, 5), Quaternion.identity);
				x += 1;
			}


			// generate until we have at least 10 levels

			while(currentLevel < 15) {
				GenerateNextLevel();
			}
		}

		SoundManager.instance.Startup ();
	}
	
	// Update is called once per frame
	void Update () {
		int cameraLevel = (int)(cameraObj.transform.position.y / platformDistance);

		if( currentLevel - cameraLevel < 10 ) {
			DestroyLastLevel();
			GenerateNextLevel ();
		}

		for( int i = 0; i < backgrounds.Length; ++i ) {
			if( cameraObj.transform.position.y - backgrounds[i].transform.position.y > backgroundUnits ) {
				backgrounds[i].transform.position += new Vector3(0, backgroundUnits * 2, 0);
			}
		}
	}

	public ArrangementInfo.Type GetCurrentLevel()
	{
		if( currentLevel > 280 ) {
			return ArrangementInfo.Type.Impossible;
		}
		if( currentLevel > 180 ) {
			return ArrangementInfo.Type.Brutal;
		}
		if( currentLevel > 120 ) {
			return ArrangementInfo.Type.Hard;
		}
		if( currentLevel > 60 ) {
			return ArrangementInfo.Type.Medium;
		}

		return ArrangementInfo.Type.Easy;
	}


	void GenerateNextLevel()
	{
		ArrangementInfo.Type useType = GetCurrentLevel();

		int count = 0;
		for( int i = 0; i < arrangements.Length; ++i ) {
			if( arrangements[i].type == useType ) {
				count += 1;
			}
		}

		if( count == 0 ) {
			useType = ArrangementInfo.Type.Easy;
			count = 1;
		}

		if( count <= arrangements.Length ) {
			int which = Random.Range (0, count);
			for( int i = 0; i < arrangements.Length; ++i ) {
				if( arrangements[i].type == useType && which-- == 0 ) {
					float position = ((arrangements[i].levels / 2.0f) + currentLevel) * platformDistance + startHeight + platformDistance;
					GameObject arrangement = (GameObject)Instantiate (arrangements[i].prefabArrangement, new Vector3(0, position, 5), Quaternion.identity);
					arrangement.GetComponent<Arrangement>().Build();
					currentLevel += arrangements[i].levels;

					usedArrangements.Enqueue(arrangement);

					break;
				}
			}
		}
		else {
			currentLevel = 9999;
		}
	}

	void DestroyLastLevel()
	{
		if( usedArrangements.Count > 0 ) {
			GameObject lastArrangement = usedArrangements.Dequeue();
			DestroyObject (lastArrangement);
		}
	}
}
